@@ -2,23 +2,7 @@
 # include <config.h>
 #endif
 
-#undef alloca
-#ifdef HAVE_ALLOCA_H
-# include <alloca.h>
-#elif defined __GNUC__
-# define alloca __builtin_alloca
-#elif defined _AIX
-# define alloca __alloca
-#elif defined _MSC_VER
-# include <malloc.h>
-# define alloca _alloca
-#else
-# include <stddef.h>
-# ifdef  __cplusplus
-extern "C"
-# endif
-void *alloca (size_t);
-#endif
+#include "efreet_alloca.h"
 
 #include <Ecore.h>
 #include <Ecore_File.h>
@@ -155,6 +139,8 @@ efreet_icon_extension_add(const char *ext)
 {
     Eina_List *l;
 
+    EINA_SAFETY_ON_NULL_RETURN(ext);
+
     ext = eina_stringshare_add(ext);
 
     if ((l = eina_list_data_find_list(efreet_icon_extensions, ext)))
@@ -235,6 +221,8 @@ efreet_icon_path_find(const char *theme_name, const char *icon, unsigned int siz
     const char *value = NULL;
     Efreet_Icon_Theme *theme;
 
+    EINA_SAFETY_ON_NULL_RETURN_VAL(icon, NULL);
+
     theme = efreet_icon_theme_find(theme_name);
 
 #ifdef SLOPPY_SPEC
@@ -281,6 +269,8 @@ efreet_icon_list_find(const char *theme_name, Eina_List *icons,
     char *data;
     Efreet_Icon_Theme *theme;
 
+    EINA_SAFETY_ON_NULL_RETURN_VAL(icons, NULL);
+
     theme = efreet_icon_theme_find(theme_name);
 
 #ifdef SLOPPY_SPEC
@@ -350,6 +340,8 @@ efreet_icon_find(const char *theme_name, const char *icon, unsigned int size)
 {
     const char *path;
 
+    EINA_SAFETY_ON_NULL_RETURN_VAL(icon, NULL);
+
     path = efreet_icon_path_find(theme_name, icon, size);
     if (path)
     {
