@@ -1,6 +1,13 @@
 #ifndef EFREET_PRIVATE_H
 #define EFREET_PRIVATE_H
 
+#ifdef ENABLE_NLS
+# include <libintl.h>
+# define _(str) dgettext(PACKAGE, str)
+#else
+# define _(str) (str)
+#endif
+
 /**
  * @file efreet_private.h
  * @brief Contains methods and defines that are private to the Efreet
@@ -179,6 +186,7 @@ int efreet_util_init(void);
 int efreet_util_shutdown(void);
 
 const char *efreet_home_dir_get(void);
+void        efreet_dirs_reset(void);
 
 const char *efreet_lang_get(void);
 const char *efreet_lang_country_get(void);
@@ -186,9 +194,8 @@ const char *efreet_lang_modifier_get(void);
 
 size_t efreet_array_cat(char *buffer, size_t size, const char *strs[]);
 
-const char *efreet_desktop_environment_get(void);
-
 void efreet_cache_desktop_update(void);
+void efreet_cache_desktop_close(void);
 void efreet_cache_icon_update(void);
 
 Efreet_Desktop *efreet_cache_desktop_find(const char *file);
