@@ -1,4 +1,3 @@
-/* vim: set sw=4 ts=4 sts=4 et: */
 #ifndef EFREET_XML_H
 #define EFREET_XML_H
 
@@ -37,7 +36,7 @@ typedef struct Efreet_Xml Efreet_Xml;
  */
 struct Efreet_Xml
 {
-    char *text;                         /**< The XML text for this node */
+    const char *text;                   /**< The XML text for this node */
     const char *tag;                    /**< The tag for this node */
 
     Efreet_Xml_Attribute **attributes;  /**< The attributes for this node */
